Package: prediction
Type: Package
Title: Tidy, Type-Safe 'prediction()' Methods
Description: A one-function package containing prediction(), a type-safe alternative to predict() that always returns a data frame. The summary() method provides a data frame with average predictions, possibly over counterfactual versions of the data (à la the margins command in 'Stata'). Marginal effect estimation is provided by the related package, 'margins' <https://cran.r-project.org/package=margins>. The package currently supports common model types (e.g., lm, glm) from the 'stats' package, as well as numerous other model classes from other add-on packages. See the README file or main package documentation page for a complete listing.
License: MIT + file LICENSE
Version: 0.3.18
Authors@R: c(person("Thomas J.", "Leeper",
                    role = c("aut"), 
                    comment = c(ORCID = "0000-0003-4097-6326")),
             person("Carl", "Ganz", role = "ctb"),
             person("Vincent", "Arel-Bundock", role = "ctb",
                    comment = c(ORCID = "0000-0003-2042-7063")),
             person("Ben","Bolker",email="bolker@mcmaster.ca", 
                    role=c("ctb","cre"),
                    comment=c(ORCID="0000-0002-2127-0443"))
            )
URL: https://github.com/bbolker/prediction
BugReports: https://github.com/bbolker/prediction/issues
Depends: R (>= 3.5.0)
Imports: utils, stats, data.table
Suggests: datasets, methods, testthat
Enhances: AER, aod, betareg, biglm, brglm, caret, crch, e1071, earth,
        ff, gam (>= 1.15), gee, glmnet, glmx, kernlab, lme4, MASS,
        mclogit, mda, mlogit, MNP, nlme, nnet, ordinal, plm, pscl,
        quantreg, rpart, sampleSelection, speedglm, survey (>= 3.31-5),
        survival, truncreg, VGAM
ByteCompile: true
Encoding: UTF-8
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-06-11 13:27:53 UTC; bolker
Author: Thomas J. Leeper [aut] (<https://orcid.org/0000-0003-4097-6326>),
  Carl Ganz [ctb],
  Vincent Arel-Bundock [ctb] (<https://orcid.org/0000-0003-2042-7063>),
  Ben Bolker [ctb, cre] (<https://orcid.org/0000-0002-2127-0443>)
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2024-06-11 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 14:50:04 UTC; unix
