# Generated by roxygen2: do not edit by hand

S3method(find_data,crch)
S3method(find_data,data.frame)
S3method(find_data,default)
S3method(find_data,glimML)
S3method(find_data,glimQL)
S3method(find_data,glm)
S3method(find_data,hxlr)
S3method(find_data,lm)
S3method(find_data,mca)
S3method(find_data,merMod)
S3method(find_data,svyglm)
S3method(find_data,train)
S3method(find_data,vgam)
S3method(find_data,vglm)
S3method(head,prediction)
S3method(mean_or_mode,data.frame)
S3method(mean_or_mode,default)
S3method(mean_or_mode,numeric)
S3method(median_or_mode,data.frame)
S3method(median_or_mode,default)
S3method(median_or_mode,numeric)
S3method(prediction,Arima)
S3method(prediction,Gam)
S3method(prediction,ar)
S3method(prediction,arima0)
S3method(prediction,betareg)
S3method(prediction,bigLm)
S3method(prediction,bigglm)
S3method(prediction,biglm)
S3method(prediction,bruto)
S3method(prediction,clm)
S3method(prediction,coxph)
S3method(prediction,crch)
S3method(prediction,default)
S3method(prediction,earth)
S3method(prediction,fda)
S3method(prediction,gausspr)
S3method(prediction,gee)
S3method(prediction,glimML)
S3method(prediction,glimQL)
S3method(prediction,glm)
S3method(prediction,glmnet)
S3method(prediction,glmx)
S3method(prediction,gls)
S3method(prediction,hetglm)
S3method(prediction,hurdle)
S3method(prediction,hxlr)
S3method(prediction,ivreg)
S3method(prediction,knnreg)
S3method(prediction,kqr)
S3method(prediction,ksvm)
S3method(prediction,lda)
S3method(prediction,lm)
S3method(prediction,lme)
S3method(prediction,loess)
S3method(prediction,lqs)
S3method(prediction,mars)
S3method(prediction,mca)
S3method(prediction,mclogit)
S3method(prediction,merMod)
S3method(prediction,mlogit)
S3method(prediction,mnlogit)
S3method(prediction,mnp)
S3method(prediction,multinom)
S3method(prediction,naiveBayes)
S3method(prediction,nls)
S3method(prediction,nnet)
S3method(prediction,plm)
S3method(prediction,polr)
S3method(prediction,polyreg)
S3method(prediction,ppr)
S3method(prediction,princomp)
S3method(prediction,qda)
S3method(prediction,rlm)
S3method(prediction,rpart)
S3method(prediction,rq)
S3method(prediction,selection)
S3method(prediction,speedglm)
S3method(prediction,speedlm)
S3method(prediction,survreg)
S3method(prediction,svm)
S3method(prediction,svyglm)
S3method(prediction,train)
S3method(prediction,tree)
S3method(prediction,truncreg)
S3method(prediction,vgam)
S3method(prediction,vglm)
S3method(prediction,zeroinfl)
S3method(print,prediction)
S3method(print,summary.prediction)
S3method(summary,prediction)
S3method(tail,prediction)
export(build_datalist)
export(find_data)
export(mean_or_mode)
export(median_or_mode)
export(prediction)
export(prediction_summary)
export(seq_range)
import(stats)
importFrom(data.table,rbindlist)
importFrom(stats,model.frame)
importFrom(stats,terms)
importFrom(utils,head)
importFrom(utils,tail)
